% reading and plotting data from arduino
% manually settings necessary
%
% function: data = arduinoGraf(port, timeout)
% port:     string name of arduino port
% timeout:  data read time
% data:     readed data

function data = arduinoGraf(port, timeout)

fd=readFromArduino(port); %open port
fd

figure; % SET UP FIGURE
  hold on;
  l=2000;
  xlim([0 l]);
  ylim([0 300]);

pause(0.1); % don't remove this pause

finishup = onCleanup(@() cleanupFun(fd)); %close port after ctrl+c interrupt

start_time=seconds();
K=0;
data=zeros(l,1);
dataLength = 0;

while true
    for i=dataLength+1 : dataLength+l
        % wait for synchronization sequence
        % [255, 255, 255, 255]
        % it must be the same as in Simulink model
        sync=zeros(1,4);
        j=1;
        while sum(sync==255)~=4           
            sync(j)=readFromArduino(fd,1);
            j=j+1;                        
            if j==5
                j=1;
            end
        end
        
        % read data
        % NUMBER OF BYTES MUST BE THE SAME AS IN SIMULINK MODEL
        a=readFromArduino(fd,2);
        % CONVERT DATA TYPES MANUALLY
        data(i,1)=typecast(uint8(a(1:2)),'int16');
        
        % real time data plot, each Kth value, low K can delay plotting
        % SET PLOT MANUALLY
        if K==100
            plot(i-dataLength,data(i,1),'r.');
            K=0;
        end
        K=K+1;
        pause(1e-6); % don't remove this pause
        if seconds()-start_time>timeout
            readFromArduino(fd); % close port after timeout
            return;              % and end function
        end
    end
    dataLength=length(data(:,1));
    data(dataLength+1 : dataLength+l,:) = 0;
    cla % clear figure
end
end

function s=seconds() % return current time in seconds
    c=clock;
    s=c(6)+60*c(5)+60*60*c(4);
end

function cleanupFun(fd)
    readFromArduino(fd);
end










