# ifndef MATLAB_MEX_FILE

# ifndef ENCODER_COMPUTE
# define ENCODER_COMPUTE

# include <Arduino.h>

typedef struct {
  long pos;
  long old_pos;
  long time;
  long old_time;
  double vel;
} Encoder;

double T;
volatile Encoder encoder_0 = {0,0,0,0,0};
volatile Encoder encoder_1 = {0,0,0,0,0};

/* interrupt routines */
void update_pos_A0() {
  encoder_0.pos++;
  encoder_0.time = micros();
}
void update_pos_B0() {
  encoder_0.pos--;
  encoder_0.time = micros();
}
void update_pos_A1() {
  encoder_1.pos++;
  encoder_1.time = micros();
}
void update_pos_B1() {
  encoder_1.pos--;
  encoder_1.time = micros();
}

/* set interrupt pins */
void init_encoder(int pA0, int pB0, int pA1, int pB1, double sample_time){
  T = sample_time;
  attachInterrupt(pA0, update_pos_A0, RISING);
  attachInterrupt(pB0, update_pos_B0, RISING);
  attachInterrupt(pA1, update_pos_A1, RISING);
  attachInterrupt(pB1, update_pos_B1, RISING);
}

/* velocity in pulses per seconds */
volatile double compute_velocity(Encoder* enc) {
  long D = enc->pos - enc->old_pos;
  if(D != 0) {
    enc->vel = 1000000.0 * D/(enc->time-enc->old_time);
    enc->old_pos = enc->pos;
    enc->old_time = enc->time;
  } else {
    int s;
    if(enc->vel < 0) s = -1; else s = 1;
    enc->vel = s * 1000000.0 / (micros()-enc->old_time);
  }
  return enc->vel;
}
# endif
# endif

